/* BASEC.C - Predict class targets using base rates. */

#include <stdio.h>
#include <stdlib.h>

/* PROGRAM TO PREDICT CLASS TARGETS USING BASE RATES.

   Usage:

      basec #classes instance

   Reads cases from train.n, where n is the instance number given as the
   second argument, ignoring all but the last number on each line, which 
   should be the class of that training case (a number from 0 up to the 
   number of classes minus 1).  Also reads a file of inputs for test cases
   from test.n, which it completely ignores, except to count how many test 
   cases there are.  

   Writes two files, each having as many lines are there are test cases,   
   with each line being the same.  The lines in the file cguess.n contain 
   the most frequent class from the training data, with ties resolved by 
   picking the lower-numbered class.  This is a reasonable guess for 0-1
   loss.  The lines in the file prob.n contain the probabilities of the 
   classes, estimated by the frequencies of the classes in the training set. 

   The probability for a class can be zero, if the class does not occur
   as the target for any case in the training set.  This may make this
   method unsuitable when log probability loss is being used, as the
   loss can be infinite. */

int main
( int argc,
  char **argv
)
{
  int nclasses, instance;

  char filename[100];  
  FILE *f;

  int train_cases;
  int test_cases;

  char item[101];
  int n;
  
  int *counts;
  int guess;
  int i;

  /* Look at arguments. */

  if (argc!=3 
   || (nclasses=atoi(argv[1])) < 1 
   || (instance=atoi(argv[2])) <= 0 && strcmp(argv[2],"0")!=0)
  { fprintf(stderr, "Usage: basec #classes instance\n");
    exit(1);
  }

  /* Read training data and accumulate counts. */

  sprintf(filename,"train.%d",instance);
  f = fopen(filename,"r");
  if (f==NULL)
  { fprintf(stderr,"Can't open file of training cases: %s\n",filename);
    exit(1);
  }

  counts = calloc(nclasses, sizeof *counts);
  if (counts==0) 
  { fprintf(stderr,"Not enough memory to hold counts");
    exit(1);
  }

  for (i = 0; i<nclasses; i++) counts[i] = 0;
  train_cases = 0; 

  for (;;)
  { 
    switch (read_last_item(f,item,100))
    { 
      case EOF: 
      { goto eof;
      }

      case 0:
      { 
        if (sscanf(item,"%d",&n)!=1)
        { fprintf(stderr,"Bad line in training data file\n");
          exit(1);
        }

        if (n<0 || n>=nclasses)
        { fprintf(stderr,"Target value of %d not in range of 0 to %d\n",
                  n, nclasses-1);
          exit(1);
        }

        train_cases += 1;
        counts[n] += 1;

        break;
      }

      default:
      { fprintf(stderr,"Bad line in training data file\n");
        exit(1);
      }
    }
  }

eof:
  fclose(f);

  /* See how many test cases there are. */

  sprintf(filename,"test.%d",instance);
  f = fopen(filename,"r");
  if (f==NULL)
  { fprintf(stderr,"Can't open file of test cases: %s\n",filename);
    exit(1);
  }

  test_cases = count_lines(f);

  fclose(f);

  /* Find most likely class. */

  guess = 0;
  for (i = 1; i<nclasses; i++)
  { if (counts[i]>counts[guess]) guess = i;
  }

  /* Write file of guesses. */

  sprintf(filename,"cguess.%d",instance);
  f = fopen(filename,"w");
  if (f==NULL)
  { fprintf(stderr,"Can't create file of guesses: %s\n",filename);
    exit(1);
  }

  for (i = 0; i<test_cases; i++)
  { fprintf(f,"%3d\n",guess);
  }

  fclose(f);

  /* Write file of probabilities. */

  sprintf(filename,"prob.%d",instance);
  f = fopen(filename,"w");
  if (f==NULL)
  { fprintf(stderr,"Can't create file of probabilities: %s\n",filename);
    exit(1);
  }

  for (i = 0; i<test_cases; i++)
  { for (n = 0; n<nclasses; n++)
    { fprintf (f, " %7.5lf", (double)counts[n] / train_cases);
    }
    fprintf(f,"\n");
  }

  fclose(f);

  /* Display distribution and best guess on standard error. */

  fprintf(stderr,"Instance %d: probs=",instance);
  for (n = 0; n<nclasses; n++) 
  { fprintf (stderr, "%.3lf ", (double)counts[n] / train_cases);
  }
  fprintf(stderr,"guess=%d\n",guess);

  exit(0);
}
