/* BASER.C - Predict numeric targets using mean and median. */

#include <stdio.h>
#include <stdlib.h>

/* PROGRAM TO DO BASE-LINE PREDICTION FOR NUMERIC TARGETS USING MEAN AND MEDIAN.

   Usage:

      baser instance

   Reads cases from train.n, where n is the instance number given as the
   argument, ignoring all but the last number on each line, which 
   should be the target in that training case (a number).  Also reads a 
   file of inputs for test cases from test.n, which it completely ignores, 
   except to count how many test cases there are.  

   Writes two files, each having as many lines are there are test cases,   
   with each line being the same.  The lines in the file cguess.S.n contain 
   the most mean of the training targets, which is a reasonable guess for 
   squared-error loss.  The lines in the file cguess.A.n contain the 
   median of the training targets, which is a reasonable guess for 
   absolute-error loss.  When the number of targets is even, the median is 
   the average of the two middle targets. 

   This method does not produce the predictive distributions that would be 
   required for evaluation by log probability loss. */

static int compare (const void *a, const void *b);

int main
( int argc,
  char **argv
)
{
  int instance;

  char filename[100];  
  FILE *f;

  int train_cases;
  int test_cases;

  char item[101];
  
  double *targets, mean, median;
  int i;

  /* Look at arguments. */

  if (argc!=2 
   || (instance=atoi(argv[1])) <= 0 && strcmp(argv[1],"0")!=0)
  { fprintf(stderr, "Usage: baser instance\n");
    exit(1);
  }

  /* Count training cases, then allocate space for targets. */

  sprintf(filename,"train.%d",instance);
  f = fopen(filename,"r");
  if (f==NULL)
  { fprintf(stderr,"Can't open file of training cases: %s\n",filename);
    exit(1);
  }

  train_cases  = count_lines(f);

  fclose(f);

  targets = calloc(train_cases, sizeof *targets);
  if (targets==0) 
  { fprintf(stderr,"Not enough memory to hold targets");
    exit(1);
  }

  /* Read training cases, storing targets. */

  sprintf(filename,"train.%d",instance);
  f = fopen(filename,"r");
  if (f==NULL)
  { fprintf(stderr,"Can't open file of training cases: %s\n",filename);
    exit(1);
  }

  for (i = 0; i<train_cases; i++)
  { 
    switch (read_last_item(f,item,100))
    { 
      case EOF: 
      { fprintf(stderr,
         "Strange behaviour: Number of training cases is changing!\n");
        exit(1);
      }

      case 0:
      { 
        if (sscanf(item,"%lf",&targets[i])!=1)
        { fprintf(stderr,"Bad line in training data file\n");
          exit(1);
        }

        break;
      }

      default:
      { fprintf(stderr,"Bad line in training data file\n");
        exit(1);
      }
    }

  }

  if (read_last_item(f,item,100)!=EOF)
  { fprintf(stderr,
     "Strange behaviour: Number of training cases is changing!\n");
    exit(1);
  }

  fclose(f);

  /* See how many test cases there are. */

  sprintf(filename,"test.%d",instance);
  f = fopen(filename,"r");
  if (f==NULL)
  { fprintf(stderr,"Can't open file of test cases: %s\n",filename);
    exit(1);
  }

  test_cases = count_lines(f);

  fclose(f);

  /* Find mean of targets. */

  mean = 0;
  for (i = 0; i<train_cases; i++)
  { mean += targets[i];
  }

  mean /= train_cases;

  /* Find median of targets. */

  qsort (targets, train_cases, sizeof *targets, compare);

  median = train_cases%2==1 ? targets[train_cases/2]
            : (targets[train_cases/2 - 1] + targets[train_cases/2]) / 2;

  /* Write files of guesses. */

  sprintf(filename,"cguess.S.%d",instance);
  f = fopen(filename,"w");
  if (f==NULL)
  { fprintf(stderr,
     "Can't create file of squared-error guesses: %s\n",filename);
    exit(1);
  }

  for (i = 0; i<test_cases; i++)
  { fprintf(f,"%.6le\n",mean);
  }

  fclose(f);

  sprintf(filename,"cguess.A.%d",instance);
  f = fopen(filename,"w");
  if (f==NULL)
  { fprintf(stderr,
     "Can't create file of absolute-error guesses: %s\n",filename);
    exit(1);
  }

  for (i = 0; i<test_cases; i++)
  { fprintf(f,"%.6le\n",median);
  }

  fclose(f);

  /* Display mean and median on standard error. */

  fprintf(stderr,"Instance %d: mean=%.6lf, median=%.6lf\n",
    instance, mean, median);

  exit(0);
}


/* COMPARE TWO DOUBLES. */

static int compare (const void *a, const void *b)
{
  return *(double*)a > *(double*)b ? 1 : *(double*)a < *(double*)b ? -1 : 0;
}
